package org.processmining.causalactivitygraphcreator.creators;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.processmining.causalactivitygraphcreator.creators.impl.GraphConnectedCreator;
import org.processmining.causalactivitygraphcreator.creators.impl.GraphSplitIOCreator;
import org.processmining.causalactivitygraphcreator.creators.impl.GraphStrongestCreator;
import org.processmining.causalactivitygraphcreator.creators.impl.GraphStrongestEdgeCreator;
import org.processmining.causalactivitygraphcreator.creators.impl.GraphWeightProbabilityCreator;
import org.processmining.causalactivitygraphcreator.creators.impl.GraphZeroCreator;

public class GraphCreatorManager {

	private static GraphCreatorManager instance = null;
	private List<GraphCreator> creators;
	private GraphCreator defaultCreator;
	private boolean sorted;

	private GraphCreatorManager() {
		creators = new ArrayList<GraphCreator>();
		defaultCreator = null;
		sorted = false;
		register(new GraphZeroCreator(), true);
		register(new GraphStrongestCreator(), false);
		register(new GraphStrongestEdgeCreator(), false);
		register(new GraphConnectedCreator(), false);
		register(new GraphSplitIOCreator(), false);
		register(new GraphWeightProbabilityCreator(), false);
	}

	public static GraphCreatorManager getInstance() {
		if (instance == null) {
			instance = new GraphCreatorManager();
		}
		return instance;
	}
	
	public void register(GraphCreator creator, boolean isDefault) {
		creators.add(creator);
		if (isDefault) {
			defaultCreator = creator;
		}
		sorted = false;
	}

	public List<GraphCreator> getCreators() {
		if (!sorted) {
			Collections.sort(creators, new Comparator<GraphCreator>() {
				public int compare(GraphCreator m1, GraphCreator m2) {
					return m1.getName().compareTo(m2.getName());
				}
			});
			sorted = true;
		}
		return creators;
	}
	
	public boolean isDefault(GraphCreator miner) {
		return miner == defaultCreator;
	}
	
	public GraphCreator getCreator(String name) {
		if (name != null) {
			for (GraphCreator creator : creators) {
				if (name.equals(creator.getName())) {
					return creator;
				}
			}
		}
		return defaultCreator;
	}
}
