package org.processmining.causalactivitygraphcreator.creators.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.causalactivitygraph.models.impl.CausalActivityGraphFactory;
import org.processmining.causalactivitygraphcreator.creators.GraphCreator;
import org.processmining.causalactivitygraphcreator.creators.GraphCreatorParameters;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;
import org.processmining.framework.plugin.PluginContext;

public class GraphConnectedCreator implements GraphCreator {

	public final static String NAME = "Strongest Connected Values";

	private class Record {
		double value;
		XEventClass source;
		XEventClass target;
	}

	public CausalActivityGraph apply(PluginContext context, CausalActivityMatrix matrix,
			GraphCreatorParameters parameters) {
		CausalActivityGraph graph = CausalActivityGraphFactory.createCausalActivityGraph();
		List<XEventClass> activities = matrix.getActivities();
		graph.init(matrix.getLabel(), new HashSet<XEventClass>(activities));
		List<Record> records = new ArrayList<Record>();
		for (XEventClass source : activities) {
			for (XEventClass target : activities) {
				Record record = new Record();
				record.value = (matrix.getValue(source, target) + 1.0) / 2.0;
				record.source = source;
				record.target = target;
				records.add(record);
			}
		}
		Collections.sort(records, new Comparator<Record>() {

			public int compare(Record o1, Record o2) {
				return (o1.value > o2.value ? -1 : (o1.value < o2.value ? 1 : 0));
			}

		});
		Map<XEventClass, XEventClass> rootSourceComponent = new HashMap<XEventClass, XEventClass>();
		Map<XEventClass, XEventClass> rootTargetComponent = new HashMap<XEventClass, XEventClass>();
		for (XEventClass activity : activities) {
			rootSourceComponent.put(activity, activity);
			rootTargetComponent.put(activity, activity);
		}
		for (Record record : records) {
			XEventClass sourceActivity = rootSourceComponent.get(record.source);
			XEventClass targetActivity = rootSourceComponent.get(record.target);
			if (sourceActivity != targetActivity) {
				if (targetActivity == record.target) {
					graph.setCausality(record.source, record.target, record.value);
					for (XEventClass activity : activities) {
						if (rootSourceComponent.get(activity) == targetActivity) {
							rootSourceComponent.put(activity, sourceActivity);
						}
					}
				}
			}
			sourceActivity = rootTargetComponent.get(record.source);
			targetActivity = rootTargetComponent.get(record.target);
			if (sourceActivity != targetActivity) {
				if (sourceActivity == record.source) {
					graph.setCausality(record.source, record.target, record.value);
					for (XEventClass activity : activities) {
						if (rootTargetComponent.get(activity) == sourceActivity) {
							rootTargetComponent.put(activity, targetActivity);
						}
					}
				}
			}
		}
		return graph;
	}

	public String getName() {
		return NAME;
	}

}
