package org.processmining.causalactivitygraphcreator.plugins;

import java.util.Collection;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.causalactivitygraphcreator.algorithms.GraphCreatorsAlgorithm;
import org.processmining.causalactivitygraphcreator.connections.GraphCreatorsConnection;
import org.processmining.causalactivitygraphcreator.dialogs.GraphCreatorsDialog;
import org.processmining.causalactivitygraphcreator.help.GraphCreatorsHelp;
import org.processmining.causalactivitygraphcreator.parameters.GraphCreatorsParameters;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin(name = "Create Graph using Creator", parameterLabels = { "Causal Activity Matrix", "Parameters" }, returnLabels = { "Causal Activity Graph" }, returnTypes = { CausalActivityGraph.class }, help = GraphCreatorsHelp.TEXT)
public class GraphCreatorsPlugin extends GraphCreatorsAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Create Graph using Creator, UI", requiredParameterLabels = { 0 })
	public CausalActivityGraph runUI(UIPluginContext context, CausalActivityMatrix matrix) {
		GraphCreatorsParameters parameters = new GraphCreatorsParameters(matrix);
		GraphCreatorsDialog dialog = new GraphCreatorsDialog(matrix, parameters);
		InteractionResult result = context.showWizard("Configure creator", true, true, dialog);
		if (result != InteractionResult.FINISHED) {
			return null;
		}
		return runConnection(context, matrix, parameters);
	}

	/**
	 * @deprecated Use runUI() instead.
	 */
	@Deprecated
	public CausalActivityGraph convertUI(UIPluginContext context, CausalActivityMatrix matrix) {
		return runUI(context, matrix);
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Create Graph using Creator, Default", requiredParameterLabels = { 0 })
	public CausalActivityGraph runDefault(PluginContext context, CausalActivityMatrix matrix) {
		GraphCreatorsParameters parameters = new GraphCreatorsParameters(matrix);
		return runConnection(context, matrix, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public CausalActivityGraph convertDefault(PluginContext context, CausalActivityMatrix matrix) {
		return runDefault(context, matrix);
	}

	@PluginVariant(variantLabel = "Create Graph using Creator, Parameters", requiredParameterLabels = { 0 })
	public CausalActivityGraph run(PluginContext context, CausalActivityMatrix matrix,
			GraphCreatorsParameters parameters) {
		return runConnection(context, matrix, parameters);
	}

	/**
	 * @deprecated Use run() instead.
	 */
	@Deprecated
	public CausalActivityGraph convertParameters(PluginContext context, CausalActivityMatrix matrix,
			GraphCreatorsParameters parameters) {
		return run(context, matrix, parameters);
	}

	private CausalActivityGraph runConnection(PluginContext context, CausalActivityMatrix matrix,
			GraphCreatorsParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<GraphCreatorsConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(
						GraphCreatorsConnection.class, context, matrix);
				for (GraphCreatorsConnection connection : connections) {
					if (connection.getObjectWithRole(GraphCreatorsConnection.MATRIX)
							.equals(matrix) && connection.getParameters().equals(parameters)) {
						return connection
								.getObjectWithRole(GraphCreatorsConnection.GRAPH);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		CausalActivityGraph graph = apply(context, matrix, parameters);
		if (parameters.isTryConnections()) {
			context.getConnectionManager().addConnection(
					new GraphCreatorsConnection(matrix, graph, parameters));
		}
		return graph;
	}

}
